
import pygame

from updateable import Updateable

class Collideable(Updateable):
    def __init__(self, pos, state):
        if not isinstance(pos, pygame.Rect):
            raise Exception("needs pygame.Rect for collision detection")
        self.pos = pos

        self.state = state

        self.hurt_distance = 24
        self.hurt_time = 0
        self.hurt_time_elapsed = 0
        self.hurt_max = 200.0
        self.is_hurt = False
        self.hurt_vector = [0, 0]

        self.hit = 0

    def _hurt_vector_to_distance(self, vector):
        '''given: vector from hurter to hurtee'''
        # calculate length of vecotor
        len_vector = vector[0] ** 2 + vector[1] ** 2
        len_vector = len_vector ** 0.5
        # calculate resulting vector hurtee should move
        # divide components by length of vector - vector is normalized
        # multiply by distance to move - x and y components the hurtee should move
        t = [round((x / len_vector) * self.hurt_distance) for x in vector]
        return t

    def collide(self, rect):
        pass

    def get_collision_rect(self):
        return self.pos

    def get_orientation(self):
        print "must be overriden!"
        return "vertical"

    def hurt(self, vector):
        if self.is_hurt:
            return
        self.hurt_vector = self._hurt_vector_to_distance(vector)
        #print self.hurt_vector
        self.is_hurt = True
        self.hurt_time = self.hurt_max
        self.hit += 1

        #print "hurt vector: ", vector

    def hurting(self):
        return self.is_hurt

    def hit_wall(self, wall, direction):
        pass

    def update(self, delta):
        if not (self.is_hurt):
            return

        self.hurt_time -= delta
        self.hurt_time_elapsed += delta

        if self.hurt_time <= 0:
            self.hurt_time = 0
            self.hurt_time_elapsed = 0
            self.is_hurt = False

        scale_time = delta / self.hurt_max + 0.0
        #print delta, scale_time, self.hurt_vector, [x * scale_time for x in self.hurt_vector]
        self.pos.x += round(self.hurt_vector[0] * scale_time)
        self.pos.y += round(self.hurt_vector[1] * scale_time)

        self.state.handle_movement(self, True)

        level_width = 1024
        level_height = 1024

        if self.pos.bottom > (level_height - 32):
            self.pos.bottom = (level_height - 32)
        if self.pos.top < 32:
            self.pos.top = 32
        if self.pos.left < 32:
            self.pos.left = 32
        if self.pos.right > (level_width - 32):
            self.pos.right = level_width - 32

