
import pygame
import pygame.locals as pgl
import pygame.gfxdraw
import random

from drawable import Drawable
from updateable import Updateable


class Light(Drawable, Updateable):
    def __init__(self, draw_pos, radius):
        dp = draw_pos.copy()
        Drawable.__init__(self, dp)
        self.radius = radius
        self.light_color = pygame.Color(230, 230, 230, 70)

        Darkness.get_instance().add_light(self)

        self.light_pos = pygame.Rect(0, 0, 128, 128)
        self.light_count = 4
        self.light_flicker = 0

    def set_pos(self, pos):
        self.draw_pos.topleft = pos.topleft

    def update(self, delta):
        self.light_flicker += delta
        if self.light_flicker >= 250:
            self.light_flicker -= 250
            self.light_pos.x = random.randint(0, self.light_count - 1) * 128

    def draw(self, surface, viewport, game):
        #Drawable.draw(self, surface, viewport)
        #self.apply_viewport(viewport)
        x = self.draw_pos.x - viewport.x + 16
        y = self.draw_pos.y - viewport.y + 8
        pygame.gfxdraw.filled_circle(surface, x, y, self.radius, self.light_color)


        light = game.get_surfaces().get("lights.png")
        surface.blit(light, (x - self.light_pos.width / 2, y - self.light_pos.height / 2), self.light_pos)

    @staticmethod
    def create_light(draw_pos, radius):
        return Light(draw_pos, radius)


class Darkness(Updateable):
    instance = None
    def __init__(self, resolution, game):
        self.color = pygame.Color(30, 30, 60)
        #self.color = pygame.Color(180, 0, 180)
        #self.color = pygame.Color(180, 180, 180)
        self.lights = []
        self.ignore_viewport = True
        self.surface = pygame.Surface(resolution)
        self.game = game

        Darkness.instance = self

    def update(self, delta):
        for l in self.lights:
            l.update(delta)

    def draw(self, screen, viewport):
        self.surface.fill(self.color)
        for l in self.lights:
            l.draw(self.surface, viewport, self.game)

        screen.blit(self.surface, (0, 0), special_flags = pgl.BLEND_MULT)

    def add_light(self, light):
        self.lights.append(light)

    def remove_light(self, light):
        if light in self.lights:
            self.lights.remove(light)

    @staticmethod
    def get_instance():
        if not Darkness.instance:
            raise RuntimeError("Instance Requested before initialisation")
        return Darkness.instance
