import state
import game
import maploader
import viewport
import pygame
import pygame.locals as pgl

from gameobjects import player
from gameobjects import lights
from gameobjects import drawable
from gameobjects import updateable
from gameobjects import collideable
from gameobjects import eventable
from gameobjects import wall
from gameobjects import surfacedrawer
from gameobjects import spider

import random

class GameState(state.State):
    def __init__(self):
        self.updateables = []
        self.drawables = []
        self.eventables = []
        self.collideables = []
        self.enemies = []
        self.game = game.Game.get_instance()
        self.add_time = 0

        self.score = 0

        self.is_dead = False

    def death(self):
        self.is_dead = True
        state = self.manager.get_state("gameover")
        p = self.player
        #spritesheet = p.spritesheet
        self.manager.replace_state(state(self.score, p.spritesheet, p.animations))

    def update(self, delta):
        self.score += delta / 100.0
        for o in self.updateables:
            o.update(delta)

        if len(self.enemies) < 3 and self.add_time == 0:
            self.add_time = random.randint(1000, 5000)

        if self.add_time > 0:
            self.add_time -= delta

        if self.add_time <= 0 and len(self.enemies) < 3:
            self.add_time = 0
            self.add_spider(random.randint(1, 4))

        self.handle_collisions()

    def add_spider(self, spawn):
        spider_pos = self.map_o.get_object("spider_spawn%d" % spawn)
        spider_pos = pygame.Rect(spider_pos, (32, 32))

        s = spider.Spider(spider_pos, self.surface_manager, "walk", state = self)
        self.add(s)
        self.enemies.append(s)

    def get_vector(self, hurter, hurtee):
        x1, y1 = hurter.center
        x2, y2 = hurtee.center
        return [x2 - x1, y2 - y1]

    def handle_collisions(self):
        if self.is_dead:
            return
        p_a = self.player.get_attack_range()
        #player attacks - are enemies hit?
        if p_a:
            enemies = [x for x in self.enemies]
            enemies_rects = [x.get_collision_rect() for x in enemies]
            collisions = p_a.collidelistall(enemies_rects)
            if collisions:
                for e in collisions:
                    v = self.get_vector(self.player.get_player_pos_ref(), enemies_rects[e])
                    if not enemies[e].hurting():
                        enemies[e].hurt(v)
                        self.game.play_sound("hit")
        #check if player movement is valid
        self.handle_movement(self.player)
        # check if enemie behaviour is valid
        for e in self.enemies:
            self.handle_movement(e)

    def handle_movement(self, collide, debug = False):
        position = collide.get_collision_rect()
        if collide == self.player:
            for o in [x for x in self.collideables if isinstance(x, spider.Goodie)]:
                if o.get_collision_rect().colliderect(position):
                    self.player.add_goodie()
                    self.remove(o)

            for o in [x for x in self.enemies]:
                if o.get_collision_rect().colliderect(position):
                    self.player.hurt(self.get_vector(o.get_collision_rect(), self.player.get_collision_rect()))
        collide_rects = [x.get_collision_rect() for x in self.collideables]
        try:
            collide_rects.remove(position)
        except:
            pass

        collisions = position.collidelistall(collide_rects)


        if not collisions:
            return

        p_r = position
        vert = collide.get_direction() in ["up", "down"]
        direction = ""

        for w in collisions:
            wall = collide_rects[w]
            if wall == position:
                print "oops", 
                continue
            #if p_r.left < wall.right and p_r.right > wall.right and not vert:
            if p_r.left < wall.right and p_r.right > wall.right:
                p_r.left = wall.right
                direction = "left"
            #if p_r.right > wall.left and p_r.left < wall.left and not vert:
            if p_r.right > wall.left and p_r.left < wall.left:
                p_r.right = wall.left
                direction = "right"
            #if p_r.top < wall.bottom and p_r.bottom > wall.bottom and vert:
            if p_r.top < wall.bottom and p_r.bottom > wall.bottom:
                p_r.top = wall.bottom
                direction = "up"
            #if p_r.bottom > wall.top and p_r.top < wall.top and vert:
            if p_r.bottom > wall.top and p_r.top < wall.top:
                p_r.bottom = wall.top
                direction = "down"

        collide.hit_wall(wall, direction)

    def draw(self, screen):
        #screen.fill((0, 0, 0))
        self.surface.fill((0, 0, 0))
        #screen.blit(self.bg, (0, 0))
        for o in self.drawables:
            o.draw(self.surface, self.viewport)

        #self.darkness.draw(screen, self.viewport)
        x, y = -self.viewport.x, -self.viewport.y
        screen.blit(self.surface, (x, y))
        self.darkness.draw(screen, self.viewport)

        self.player.draw_weapons(screen)
        self.player.draw_bar(screen)

        pygame.display.flip()

    def handle_events(self, events):
        for o in self.eventables:
            o.handle_events(events)

        if events.escape:
            #self.manager.pop_state()
            pause = self.manager.get_state("pause")
            self.manager.push_state(pause())

    def add(self, obj):
        if isinstance(obj, drawable.Drawable):
            self.drawables.append(obj)
        if isinstance(obj, eventable.Eventable):
            self.eventables.append(obj)
        if isinstance(obj, updateable.Updateable):
            self.updateables.append(obj)
        if isinstance(obj, collideable.Collideable):
            self.collideables.append(obj)

    def remove(self, obj):
        if obj in self.drawables:
            self.drawables.remove(obj)
        if obj in self.updateables:
            self.updateables.remove(obj)
        if obj in self.collideables:
            self.collideables.remove(obj)
        if obj in self.eventables:
            self.eventables.remove(obj)

        if obj in self.enemies:
            self.enemies.remove(obj)


    def start(self, manager, resolution):
        self.manager = manager
        self.resolution = resolution
        self.darkness = lights.Darkness(resolution, self.game)
        self.add(self.darkness)

        self.map_o = maploader.MapObject("test.tmx")
        #self.map_o = maploader.MapObject("big.tmx")
        self.bg = self.map_o.get_surface()
        map_sf = surfacedrawer.SurfaceDrawer(pygame.Rect(0, 0, 2, 2), self.bg)
        self.add(map_sf)

        x, y = self.map_o.get_dimensions()
        self.surface = pygame.Surface((x, y))

        self.player = player.Player(self.game, self.map_o.get_player(), state = self)

        surfacemanager = self.game.get_surfaces()

        self.surface_manager = surfacemanager

        for i in xrange(1, 5):
            self.add_spider(i)

        self.add(self.player)
        #self.collideables.remove(self.player)

        self.lights = []
        #self.lights.append(lights.Light.create_light(pygame.Rect(100, 100, 32, 32), 128))

        self.viewport = viewport.Viewport(x, y, self.game.get_resolution(), self.player.get_player_pos_ref())

        self.add(self.viewport)

        self.walls = self.map_o.get_walls()
        for w in self.walls:
            self.add(wall.Wall(w, self))
            #self.add(drawable.DebugDraw(w))

    def end(self):
        self.collideables = []
        self.drawables = []
        self.updateables = []
        self.eventables = []

        self.player = None
        self.lights = None
        self.viewport = None
        self.manager = None
        self.resolution = None
        self.map_o = None
        self.darkness = None
        self.bg = None
        self.surface = None
